@@ -25,11 +25,11 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch dvdreadsrc title=1 ! mpegpsdemux ! a52dec ! audioresample ! audioconvert ! alsasink
- * ]| Play audio track from a dvd.
+ * gst-launch-1.0 dvdreadsrc title=1 ! mpegpsdemux ! a52dec ! audioconvert ! audioresample ! autoaudiosink
+ * ]| Play audio part of a dvd title.
  * |[
- * gst-launch filesrc location=abc.ac3 ! a52dec ! audioresample ! audioconvert ! alsasink
- * ]| Decode a stand alone file and play it.
+ * gst-launch-1.0 filesrc location=abc.ac3 ! ac3parse ! a52dec ! audioconvert ! audioresample ! autoaudiosink
+ * ]| Decode and play a stand alone AC-3 file.
  * </refsect2>
  */
 
@@ -229,6 +229,10 @@ gst_a52dec_init (GstA52Dec * a52dec)
   a52dec->state = NULL;
   a52dec->samples = NULL;
 
+  gst_audio_decoder_set_use_default_pad_acceptcaps (GST_AUDIO_DECODER_CAST
+      (a52dec), TRUE);
+  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_DECODER_SINK_PAD (a52dec));
+
   /* retrieve and intercept base class chain.
    * Quite HACKish, but that's dvd specs/caps for you,
    * since one buffer needs to be split into 2 frames */
@@ -242,10 +246,12 @@ gst_a52dec_start (GstAudioDecoder * dec)
 {
   GstA52Dec *a52dec = GST_A52DEC (dec);
   GstA52DecClass *klass;
+  static GMutex init_mutex;
 
   GST_DEBUG_OBJECT (dec, "start");
 
   klass = GST_A52DEC_CLASS (G_OBJECT_GET_CLASS (a52dec));
+  g_mutex_lock (&init_mutex);
 #if defined(A52_ACCEL_DETECT)
   a52dec->state = a52_init ();
   /* This line is just to avoid being accused of not using klass */
@@ -253,6 +259,7 @@ gst_a52dec_start (GstAudioDecoder * dec)
 #else
   a52dec->state = a52_init (klass->a52_cpuflags);
 #endif
+  g_mutex_unlock (&init_mutex);
 
   if (!a52dec->state) {
     GST_ELEMENT_ERROR (GST_ELEMENT (a52dec), LIBRARY, INIT, (NULL),
