@@ -367,6 +367,7 @@ gst_dvd_sub_dec_parse_subpic (GstDvdSubDec * dec)
                 gst_guint64_to_gdouble (GST_SECOND));
             return;
           }
+          break;
         } else {
           dec->parse_pos = NULL;
           dec->next_event_ts = GST_CLOCK_TIME_NONE;
@@ -568,7 +569,7 @@ gst_dvd_sub_dec_merge_title (GstDvdSubDec * dec, GstVideoFrame * frame)
   gint hl_top, hl_bottom;
   gint last_y;
   RLE_state state;
-  guint8 *Y_data;;
+  guint8 *Y_data;
 
   GST_DEBUG_OBJECT (dec, "Merging subtitle on frame");
 
@@ -691,8 +692,7 @@ gst_send_subtitle_frame (GstDvdSubDec * dec, GstClockTime end_ts)
   }
 
   out_buf =
-      gst_buffer_new_allocate (NULL, GST_VIDEO_INFO_SIZE (&dec->info),
-      &params);
+      gst_buffer_new_allocate (NULL, GST_VIDEO_INFO_SIZE (&dec->info), &params);
   gst_video_frame_map (&frame, &dec->info, out_buf, GST_MAP_READWRITE);
 
   data = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
@@ -891,6 +891,7 @@ gst_dvd_sub_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
       GstStructure *s = gst_caps_get_structure (peer_caps, i);
       /* Check if the peer pad support ARGB format, if yes change caps */
       if (gst_structure_has_name (s, "video/x-raw")) {
+        GstCaps *downstream_caps;
         gst_caps_unref (out_caps);
         GST_DEBUG_OBJECT (dec, "trying with ARGB");
 
@@ -900,12 +901,15 @@ gst_dvd_sub_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
             "height", G_TYPE_INT, dec->in_height,
             "framerate", GST_TYPE_FRACTION, 0, 1, NULL);
 
-        if (gst_pad_peer_query_accept_caps (dec->srcpad, out_caps)) {
+        downstream_caps = gst_pad_peer_query_caps (dec->srcpad, NULL);
+        if (gst_caps_can_intersect (downstream_caps, out_caps)) {
+          gst_caps_unref (downstream_caps);
           GST_DEBUG_OBJECT (dec, "peer accepted ARGB");
           /* If ARGB format then set the flag */
           dec->use_ARGB = TRUE;
           break;
         }
+        gst_caps_unref (downstream_caps);
       }
     }
     gst_caps_unref (peer_caps);
