@@ -25,16 +25,14 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch filesrc location=interview.ra ! rademux ! ffdec_real_288 ! audioconvert ! audioresample ! alsasink
+ * gst-launch-1.0 filesrc location=interview.ra ! rademux ! avdec_real_288 ! audioconvert ! audioresample ! autoaudiosink
  * ]| Read a RealAudio file and decode it and output it to the soundcard using
  * the ALSA element. The .ra file is assumed to contain RealAudio version 2.
  * |[
- * gst-launch gnomevfssrc location=http://www.example.org/interview.ra ! rademux ! a52dec ! audioconvert ! audioresample ! alsasink
+ * gst-launch-1.0 souphttpsrc location=http://www.example.org/interview.ra ! rademux ! ac3parse ! a52dec ! audioconvert ! audioresample ! autoaudiosink
  * ]| Stream RealAudio data containing AC3 (dnet) compressed audio and decode it
- * and output it to the soundcard using the ALSA element.
+ * and output it to the soundcard.
  * </refsect2>
- *
- * Last reviewed on 2006-10-24 (0.10.5)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -523,8 +521,7 @@ gst_real_audio_demux_parse_header (GstRealAudioDemux * demux)
 unknown_fourcc:
   {
     GST_ELEMENT_ERROR (GST_ELEMENT (demux), STREAM, DECODE, (NULL),
-        ("Unknown fourcc '%" GST_FOURCC_FORMAT "'",
-            GST_FOURCC_ARGS (demux->fourcc)));
+        ("Unknown fourcc '0x%" G_GINT32_MODIFIER "x'", demux->fourcc));
     return GST_FLOW_ERROR;
   }
 broken_file:
